@@ -18,8 +18,12 @@ McCullagh, P. and Nelder, J.A.  1989.  "Generalized Linear Models." 2nd ed.
     Chapman & Hall, Boca Rotan.
 """
 import numpy as np
+
 from . import families
-from statsmodels.tools.decorators import cache_readonly
+
+from statsmodels.tools.decorators import (cache_readonly,
+                                          cached_data, cached_value)
+from statsmodels.compat.pandas import Appender
 
 import statsmodels.base.model as base
 import statsmodels.regression.linear_model as lm
@@ -51,17 +55,17 @@ def _check_convergence(criterion, iteration, atol, rtol):
 
 class GLM(base.LikelihoodModel):
     __doc__ = """
-    Generalized Linear Models class
+    Generalized Linear Models
 
     GLM inherits from statsmodels.base.model.LikelihoodModel
 
     Parameters
     ----------
-    endog : array-like
+    endog : array_like
         1d array of endogenous response variable.  This array can be 1d or 2d.
         Binomial family models accept a 2d array with two columns. If
         supplied, each observation is expected to be [success, failure].
-    exog : array-like
+    exog : array_like
         A nobs x k array where `nobs` is the number of observations and `k`
         is the number of regressors. An intercept is not included by default
         and should be added by the user (models specified using a formula
@@ -71,20 +75,20 @@ class GLM(base.LikelihoodModel):
         family = sm.family.Binomial()
         Each family can take a link instance as an argument.  See
         statsmodels.family.family for more information.
-    offset : array-like or None
+    offset : array_like or None
         An offset to be included in the model.  If provided, must be
         an array whose length is the number of rows in exog.
-    exposure : array-like or None
+    exposure : array_like or None
         Log(exposure) will be added to the linear prediction in the model.
         Exposure is only valid if the log link is used. If provided, it must be
         an array with the same length as endog.
-    freq_weights : array-like
+    freq_weights : array_like
         1d array of frequency weights. The default is None. If None is selected
         or a blank value, then the algorithm will replace with an array of 1's
         with length equal to the endog.
         WARNING: Using weights is not verified yet for all possible options
         and results, see Notes.
-    var_weights : array-like
+    var_weights : array_like
         1d array of variance (analytic) weights. The default is None. If None
         is selected or a blank value, then the algorithm will replace with an
         array of 1's with length equal to the endog.
@@ -101,22 +105,22 @@ class GLM(base.LikelihoodModel):
     df_resid : float
         Residual degrees of freedom is equal to the number of observation n
         minus the number of regressors p.
-    endog : array
+    endog : ndarray
         See Notes.  Note that `endog` is a reference to the data so that if
         data is already an array and it is changed, then `endog` changes
         as well.
-    exposure : array-like
+    exposure : array_like
         Include ln(exposure) in model with coefficient constrained to 1. Can
         only be used if the link is the logarithm function.
-    exog : array
+    exog : ndarray
         See Notes.  Note that `exog` is a reference to the data so that if
         data is already an array and it is changed, then `exog` changes
         as well.
-    freq_weights : array
+    freq_weights : ndarray
         See Notes. Note that `freq_weights` is a reference to the data so that
         if data is already an array and it is changed, then `freq_weights`
         changes as well.
-    var_weights : array
+    var_weights : ndarray
         See Notes. Note that `var_weights` is a reference to the data so that
         if data is already an array and it is changed, then `var_weights`
         changes as well.
@@ -125,23 +129,23 @@ class GLM(base.LikelihoodModel):
     family : family class instance
         The distribution family of the model. Can be any family in
         statsmodels.families.  Default is Gaussian.
-    mu : array
+    mu : ndarray
         The mean response of the transformed variable.  `mu` is the value of
         the inverse of the link function at lin_pred, where lin_pred is the
         linear predicted value of the WLS fit of the transformed variable.
         `mu` is only available after fit is called.  See
         statsmodels.families.family.fitted of the distribution family for more
         information.
-    n_trials : array
+    n_trials : ndarray
         See Notes. Note that `n_trials` is a reference to the data so that if
         data is already an array and it is changed, then `n_trials` changes
         as well. `n_trials` is the number of binomial trials and only available
         with that distribution. See statsmodels.families.Binomial for more
         information.
-    normalized_cov_params : array
+    normalized_cov_params : ndarray
         The p x p normalized covariance of the design / exogenous data.
         This is approximately equal to (X.T X)^(-1)
-    offset : array-like
+    offset : array_like
         Include offset in model with coefficient constrained to 1.
     scale : float
         The estimate of the scale / dispersion of the model fit.  Only
@@ -150,7 +154,7 @@ class GLM(base.LikelihoodModel):
     scaletype : str
         The scaling used for fitting the model.  This is only available after
         fit is called.  The default is None.  See GLM.fit for more information.
-    weights : array
+    weights : ndarray
         The value of the weights after the last iteration of fit.  Only
         available after fit is called.  See statsmodels.families.family for
         the specific distribution weighting functions.
@@ -195,7 +199,7 @@ class GLM(base.LikelihoodModel):
      Gaussian      x     x   x     x      x       x   x     x      x
      inv Gaussian  x     x                        x
      binomial      x     x   x     x      x       x   x           x      x
-     Poission      x     x                        x
+     Poisson       x     x                        x
      neg binomial  x     x                        x        x
      gamma         x     x                        x
      Tweedie       x     x                        x
@@ -206,7 +210,7 @@ class GLM(base.LikelihoodModel):
     Endog and exog are references so that if the data they refer to are already
     arrays and these arrays are changed, endog and exog will change.
 
-    Statsmodels supports two separte definitions of weights: frequency weights
+    statsmodels supports two separate definitions of weights: frequency weights
     and variance weights.
 
     Frequency weights produce the same results as repeating observations by the
@@ -219,11 +223,11 @@ class GLM(base.LikelihoodModel):
     assumption that that the inverse variance scales proportionally to the
     weight--an observation that is deemed more credible should have less
     variance and therefore have more weight. For the ``Poisson`` family--which
-    assumes that occurences scale proportionally with time--a natural practice
+    assumes that occurrences scale proportionally with time--a natural practice
     would be to use the amount of time as the variance weight and set ``endog``
-    to be a rate (occurrances per period of time). Similarly, using a
+    to be a rate (occurrences per period of time). Similarly, using a
     compound Poisson family, namely ``Tweedie``, makes a similar assumption
-    about the rate (or frequency) of occurences having variance proportional to
+    about the rate (or frequency) of occurrences having variance proportional to
     time.
 
     Both frequency and variance weights are verified for all basic results with
@@ -255,8 +259,9 @@ class GLM(base.LikelihoodModel):
     interpretation. The loglikelihood is not correctly specified in this case,
     and statistics based on it, such AIC or likelihood ratio tests, are not
     appropriate.
-
     """ % {'extra_params': base._missing_param_doc}
+    # Maximum number of endogenous variables when using a formula
+    _formula_max_endog = 2
 
     def __init__(self, endog, exog, family=None, offset=None,
                  exposure=None, freq_weights=None, var_weights=None,
@@ -427,7 +432,6 @@ class GLM(base.LikelihoodModel):
         score_obs : ndarray, 2d
             The first derivative of the loglikelihood function evaluated at
             params for each observation.
-
         """
 
         score_factor = self.score_factor(params, scale=scale)
@@ -450,7 +454,6 @@ class GLM(base.LikelihoodModel):
         score : ndarray_1d
             The first derivative of the loglikelihood function calculated as
             the sum of `score_obs`
-
         """
         score_factor = self.score_factor(params, scale=scale)
         return np.dot(score_factor, self.exog)
@@ -474,7 +477,6 @@ class GLM(base.LikelihoodModel):
         score_factor : ndarray_1d
             A 1d weight vector used in the calculation of the score_obs.
             The score_obs are obtained by `score_factor[:, None] * exog`
-
         """
         mu = self.predict(params)
         if scale is None:
@@ -622,7 +624,6 @@ class GLM(base.LikelihoodModel):
             The derivative of the score_obs with respect to endog. This
             can is given by `score_factor0[:, None] * exog` where
             `score_factor0` is the score_factor without the residual.
-
         """
         mu = self.predict(params)
         if scale is None:
@@ -677,7 +678,6 @@ class GLM(base.LikelihoodModel):
         Notes
         -----
         not yet verified for case with scale not equal to 1.
-
         """
 
         if exog_extra is None:
@@ -722,13 +722,13 @@ class GLM(base.LikelihoodModel):
 
     def estimate_scale(self, mu):
         """
-        Estimates the dispersion/scale.
+        Estimate the dispersion/scale.
 
         Type of scale can be chose in the fit method.
 
         Parameters
         ----------
-        mu : array
+        mu : ndarray
             mu is the mean response estimate
 
         Returns
@@ -780,7 +780,7 @@ class GLM(base.LikelihoodModel):
 
         Parameters
         ----------
-        mu : array-like
+        mu : array_like
             Fitted mean response variable
         method : str, defaults to 'brentq'
             Scipy optimizer used to solve the Pearson equation. Only brentq
@@ -795,7 +795,7 @@ class GLM(base.LikelihoodModel):
         Returns
         -------
         power : float
-            The estimated shape or power
+            The estimated shape or power.
         """
         if method == 'brentq':
             from scipy.optimize import brentq
@@ -818,14 +818,14 @@ class GLM(base.LikelihoodModel):
 
         Parameters
         ----------
-        params : array-like
+        params : array_like
             Parameters / coefficients of a GLM.
-        exog : array-like, optional
+        exog : array_like, optional
             Design / exogenous data. Is exog is None, model exog is used.
-        exposure : array-like, optional
+        exposure : array_like, optional
             Exposure time values, only can be used with the log link
             function.  See notes for details.
-        offset : array-like, optional
+        offset : array_like, optional
             Offset values.  See notes for details.
         linear : bool
             If True, returns the linear predicted values.  If False,
@@ -878,15 +878,15 @@ class GLM(base.LikelihoodModel):
     def get_distribution(self, params, scale=1, exog=None, exposure=None,
                          offset=None):
         """
-        Returns a random number generator for the predictive distribution.
+        Return a random number generator for the predictive distribution.
 
         Parameters
         ----------
-        params : array-like
+        params : array_like
             The model parameters.
         scale : scalar
             The scale parameter.
-        exog : array-like
+        exog : array_like
             The predictor variable matrix.
 
         Returns
@@ -944,26 +944,26 @@ class GLM(base.LikelihoodModel):
 
         Parameters
         ----------
-        start_params : array-like, optional
+        start_params : array_like, optional
             Initial guess of the solution for the loglikelihood maximization.
             The default is family-specific and is given by the
             ``family.starting_mu(endog)``. If start_params is given then the
             initial mean will be calculated as ``np.dot(exog, start_params)``.
         maxiter : int, optional
             Default is 100.
-        method : string
+        method : str
             Default is 'IRLS' for iteratively reweighted least squares.
             Otherwise gradient optimization is used.
         tol : float
             Convergence tolerance.  Default is 1e-8.
-        scale : string or float, optional
+        scale : str or float, optional
             `scale` can be 'X2', 'dev', or a float
             The default value is None, which uses `X2` for Gamma, Gaussian,
             and Inverse Gaussian.
             `X2` is Pearson's chi-square divided by `df_resid`.
             The default is 1 for the Binomial and Poisson families.
             `dev` is the deviance divided by df_resid
-        cov_type : string
+        cov_type : str
             The type of parameter estimate covariance matrix to compute.
         cov_kwds : dict-like
             Extra arguments for calculating the covariance of the parameter
@@ -1002,7 +1002,7 @@ class GLM(base.LikelihoodModel):
             'lstsq' and 'pinv' regularize the estimate in singular and
             near-singular cases by truncating small singular values based
             on `rcond` of the respective numpy.linalg function. 'qr' is
-            only valied for cases that are not singular nor near-singular.
+            only valid for cases that are not singular nor near-singular.
         optim_hessian : {'eim', 'oim'}, optional
             (available with scipy optimizer fits) When 'oim'--the default--the
             observed Hessian is used in fitting. 'eim' is the expected Hessian.
@@ -1016,7 +1016,6 @@ class GLM(base.LikelihoodModel):
         in future versions. If attach_wls' is true, then the final WLS
         instance of the IRLS iteration is attached to the results instance
         as `results_wls` attribute.
-
         """
         self.scaletype = scale
 
@@ -1202,29 +1201,35 @@ class GLM(base.LikelihoodModel):
         return GLMResultsWrapper(glm_results)
 
     def fit_regularized(self, method="elastic_net", alpha=0.,
-                        start_params=None, refit=False, **kwargs):
+                        start_params=None, refit=False,
+                        opt_method="bfgs", **kwargs):
         r"""
         Return a regularized fit to a linear regression model.
 
         Parameters
         ----------
-        method :
+        method : {'elastic_net'}
             Only the `elastic_net` approach is currently implemented.
-        alpha : scalar or array-like
+        alpha : scalar or array_like
             The penalty weight.  If a scalar, the same penalty weight
             applies to all variables in the model.  If a vector, it
             must have the same length as `params`, and contains a
             penalty weight for each coefficient.
-        start_params : array-like
+        start_params : array_like
             Starting values for `params`.
         refit : bool
             If True, the model is refit using only the variables that
             have non-zero coefficients in the regularized fit.  The
             refitted model is not regularized.
+        opt_method : string
+            The method used for numerical optimization.
+        **kwargs
+            Additional keyword arguments used when fitting the model.
 
         Returns
         -------
-        An array, or a GLMResults object of the same type returned by `fit`.
+        GLMResults
+            An array or a GLMResults object, same type returned by `fit`.
 
         Notes
         -----
@@ -1256,7 +1261,7 @@ class GLM(base.LikelihoodModel):
         """
 
         if kwargs.get("L1_wt", 1) == 0:
-            return self._fit_ridge(alpha, start_params)
+            return self._fit_ridge(alpha, start_params, opt_method)
 
         from statsmodels.base.elastic_net import fit_elasticnet
 
@@ -1278,13 +1283,13 @@ class GLM(base.LikelihoodModel):
 
         return result
 
-    def _fit_ridge(self, alpha, start_params, method="newton-cg"):
+    def _fit_ridge(self, alpha, start_params, method):
 
         if start_params is None:
             start_params = np.zeros(self.exog.shape[1])
 
         def fun(x):
-            return -(self.loglike(x) / self.nobs - alpha * np.sum(x**2) / 2)
+            return -(self.loglike(x) / self.nobs - np.sum(alpha * x**2) / 2)
 
         def grad(x):
             return -(self.score(x) / self.nobs - alpha * x)
@@ -1336,7 +1341,6 @@ class GLM(base.LikelihoodModel):
         Returns
         -------
         results : Results instance
-
         """
 
         from patsy import DesignInfo
@@ -1389,18 +1393,18 @@ class GLMResults(base.LikelihoodModelResults):
         Pointer to GLM model instance that called fit.
     nobs : float
         The number of observations n.
-    normalized_cov_params : array
+    normalized_cov_params : ndarray
         See GLM docstring
-    params : array
+    params : ndarray
         The coefficients of the fitted model.  Note that interpretation
         of the coefficients often depends on the distribution family and the
         data.
-    pvalues : array
+    pvalues : ndarray
         The two-tailed p-values for the parameters.
     scale : float
         The estimate of the scale / dispersion for the model fit.
         See GLM.fit and GLM.estimate_scale for more information.
-    stand_errors : array
+    stand_errors : ndarray
         The standard errors of the fitted GLM.   #TODO still named bse
 
     See Also
@@ -1472,7 +1476,7 @@ class GLMResults(base.LikelihoodModelResults):
             get_robustcov_results(self, cov_type=cov_type, use_self=True,
                                   use_t=use_t, **cov_kwds)
 
-    @cache_readonly
+    @cached_data
     def resid_response(self):
         """
         Respnose residuals.  The response residuals are defined as
@@ -1480,7 +1484,7 @@ class GLMResults(base.LikelihoodModelResults):
         """
         return self._n_trials * (self._endog-self.mu)
 
-    @cache_readonly
+    @cached_data
     def resid_pearson(self):
         """
         Pearson residuals.  The Pearson residuals are defined as
@@ -1492,7 +1496,7 @@ class GLMResults(base.LikelihoodModelResults):
                 np.sqrt(self._var_weights) /
                 np.sqrt(self.family.variance(self.mu)))
 
-    @cache_readonly
+    @cached_data
     def resid_working(self):
         """
         Working residuals.  The working residuals are defined as
@@ -1504,7 +1508,7 @@ class GLMResults(base.LikelihoodModelResults):
         val *= self._n_trials
         return val
 
-    @cache_readonly
+    @cached_data
     def resid_anscombe(self):
         """
         Anscombe residuals.  See statsmodels.families.family for distribution-
@@ -1518,7 +1522,7 @@ class GLMResults(base.LikelihoodModelResults):
                                           var_weights=self._var_weights,
                                           scale=1.)
 
-    @cache_readonly
+    @cached_data
     def resid_anscombe_scaled(self):
         """
         Scaled Anscombe residuals.  See statsmodels.families.family for
@@ -1528,7 +1532,7 @@ class GLMResults(base.LikelihoodModelResults):
                                           var_weights=self._var_weights,
                                           scale=self.scale)
 
-    @cache_readonly
+    @cached_data
     def resid_anscombe_unscaled(self):
         """
         Unscaled Anscombe residuals.  See statsmodels.families.family for
@@ -1538,7 +1542,7 @@ class GLMResults(base.LikelihoodModelResults):
                                           var_weights=self._var_weights,
                                           scale=1.)
 
-    @cache_readonly
+    @cached_data
     def resid_deviance(self):
         """
         Deviance residuals.  See statsmodels.families.family for distribution-
@@ -1549,7 +1553,7 @@ class GLMResults(base.LikelihoodModelResults):
                                     scale=1.)
         return dev
 
-    @cache_readonly
+    @cached_value
     def pearson_chi2(self):
         """
         Pearson's Chi-Squared statistic is defined as the sum of the squares
@@ -1560,7 +1564,7 @@ class GLMResults(base.LikelihoodModelResults):
         chisqsum = np.sum(chisq)
         return chisqsum
 
-    @cache_readonly
+    @cached_data
     def fittedvalues(self):
         """
         Linear predicted values for the fitted model.
@@ -1568,7 +1572,7 @@ class GLMResults(base.LikelihoodModelResults):
         """
         return self.mu
 
-    @cache_readonly
+    @cached_data
     def mu(self):
         """
         See GLM docstring.
@@ -1621,7 +1625,7 @@ class GLMResults(base.LikelihoodModelResults):
                                    freq_weights=self._freq_weights,
                                    scale=self.scale)
 
-    @cache_readonly
+    @cached_value
     def llf(self):
         """
         Value of the loglikelihood function evalued at params.
@@ -1642,7 +1646,7 @@ class GLMResults(base.LikelihoodModelResults):
                                    scale=scale)
         return val
 
-    @cache_readonly
+    @cached_value
     def aic(self):
         """
         Akaike Information Criterion
@@ -1650,7 +1654,7 @@ class GLMResults(base.LikelihoodModelResults):
         """
         return -2 * self.llf + 2 * (self.df_model + 1)
 
-    @cache_readonly
+    @cached_value
     def bic(self):
         """
         Bayes Information Criterion
@@ -1660,6 +1664,7 @@ class GLMResults(base.LikelihoodModelResults):
                 (self.model.wnobs - self.df_model - 1) *
                 np.log(self.model.wnobs))
 
+    @Appender(pred.get_prediction_glm.__doc__)
     def get_prediction(self, exog=None, exposure=None, offset=None,
                        transform=True, linear=False,
                        row_labels=None):
@@ -1683,8 +1688,6 @@ class GLMResults(base.LikelihoodModelResults):
 
         return res
 
-    get_prediction.__doc__ = pred.get_prediction_glm.__doc__
-
     def get_hat_matrix_diag(self, observed=True):
         """
         Compute the diagonal of the hat matrix
@@ -1743,6 +1746,7 @@ class GLMResults(base.LikelihoodModelResults):
                          hat_matrix_diag=hat_matrix_diag)
         return infl
 
+    @Appender(base.LikelihoodModelResults.remove_data.__doc__)
     def remove_data(self):
         # GLM has alias/reference in result instance
         self._data_attr.extend([i for i in self.model._data_attr
@@ -1756,12 +1760,10 @@ class GLMResults(base.LikelihoodModelResults):
         self._iweights = None
         self._n_trials = None
 
-    remove_data.__doc__ = base.LikelihoodModelResults.remove_data.__doc__
-
+    @Appender(_plot_added_variable_doc % {'extra_params_doc': ''})
     def plot_added_variable(self, focus_exog, resid_type=None,
                             use_glm_weights=True, fit_kwargs=None,
                             ax=None):
-        # Docstring attached below
 
         from statsmodels.graphics.regressionplots import plot_added_variable
 
@@ -1772,42 +1774,35 @@ class GLMResults(base.LikelihoodModelResults):
 
         return fig
 
-    plot_added_variable.__doc__ = _plot_added_variable_doc % {
-        'extra_params_doc': ''}
-
+    @Appender(_plot_partial_residuals_doc % {'extra_params_doc': ''})
     def plot_partial_residuals(self, focus_exog, ax=None):
-        # Docstring attached below
 
         from statsmodels.graphics.regressionplots import plot_partial_residuals
 
         return plot_partial_residuals(self, focus_exog, ax=ax)
 
-    plot_partial_residuals.__doc__ = _plot_partial_residuals_doc % {
-        'extra_params_doc': ''}
-
+    @Appender(_plot_ceres_residuals_doc % {'extra_params_doc': ''})
     def plot_ceres_residuals(self, focus_exog, frac=0.66, cond_means=None,
                              ax=None):
-        # Docstring attached below
 
         from statsmodels.graphics.regressionplots import plot_ceres_residuals
 
         return plot_ceres_residuals(self, focus_exog, frac,
                                     cond_means=cond_means, ax=ax)
 
-    plot_ceres_residuals.__doc__ = _plot_ceres_residuals_doc % {
-        'extra_params_doc': ''}
-
     def summary(self, yname=None, xname=None, title=None, alpha=.05):
         """
         Summarize the Regression Results
 
         Parameters
         ----------
-        yname : string, optional
+        yname : str, optional
             Default is `y`
-        xname : list of strings, optional
-            Default is `var_##` for ## in p the number of regressors
-        title : string, optional
+        xname : list[str], optional
+            Names for the exogenous variables, default is `var_#` for ## in
+            the number of regressors. Must match the number of parameters in
+            the model
+        title : str, optional
             Title for the top table. If not None, then this replaces the
             default title
         alpha : float
@@ -1869,16 +1864,18 @@ class GLMResults(base.LikelihoodModelResults):
 
         Parameters
         ----------
-        yname : string
+        yname : str
             Name of the dependent variable (optional)
-        xname : List of strings of length equal to the number of parameters
-            Names of the independent variables (optional)
-        title : string, optional
+        xname : list[str], optional
+            Names for the exogenous variables, default is `var_#` for ## in
+            the number of regressors. Must match the number of parameters in
+            the model
+        title : str, optional
             Title for the top table. If not None, then this replaces the
             default title
         alpha : float
             significance level for the confidence intervals
-        float_format: string
+        float_format : str
             print format for floats in parameters summary
 
         Returns
@@ -1927,7 +1924,6 @@ if __name__ == "__main__":
     # print(GLMT[0])
     # print(GLMT[2])
     GLMTp = GLMmod.summary(title='Test GLM')
-
     """
 From Stata
 . webuse beetle
